/// <reference types="typed-graphql" />
/// <reference types="@types/node" />
import { GraphQLSchema, GraphQLResolveInfo, GraphQLFieldResolveFn } from 'graphql';
import { IExecutableSchemaDefinition, ILogger, IResolvers, ITypeDefinitions, IFieldIteratorFn, IResolverValidationOptions } from './Interfaces';
declare class SchemaError extends Error {
    message: string;
    constructor(message: string);
}
declare function makeExecutableSchema({typeDefs, resolvers, connectors, logger, allowUndefinedInResolve, resolverValidationOptions}: IExecutableSchemaDefinition): GraphQLSchema;
declare function buildSchemaFromTypeDefinitions(typeDefinitions: ITypeDefinitions): GraphQLSchema;
declare function forEachField(schema: GraphQLSchema, fn: IFieldIteratorFn): void;
declare const attachConnectorsToContext: Function;
declare function addSchemaLevelResolveFunction(schema: GraphQLSchema, fn: GraphQLFieldResolveFn): void;
declare function addResolveFunctionsToSchema(schema: GraphQLSchema, resolveFunctions: IResolvers): void;
declare function assertResolveFunctionsPresent(schema: GraphQLSchema, resolverValidationOptions?: IResolverValidationOptions): void;
declare function addErrorLoggingToSchema(schema: GraphQLSchema, logger: ILogger): void;
declare function chainResolvers(resolvers: GraphQLFieldResolveFn[]): (root: any, args: {
    [argName: string]: any;
}, ctx: any, info: GraphQLResolveInfo) => any;
declare function addCatchUndefinedToSchema(schema: GraphQLSchema): void;
export { makeExecutableSchema, SchemaError, forEachField, chainResolvers, addErrorLoggingToSchema, addResolveFunctionsToSchema, addCatchUndefinedToSchema, assertResolveFunctionsPresent, buildSchemaFromTypeDefinitions, addSchemaLevelResolveFunction, attachConnectorsToContext };
